@@ -15,7 +15,6 @@ from tokenize import PseudoToken
 
 from Cheetah import five
 from Cheetah.SourceReader import SourceReader
-from Cheetah.Unspecified import Unspecified
 
 if five.PY2:  # pragma: no cover (PY2 only)
     from itertools import izip_longest  # pylint:disable=no-name-in-module
@@ -24,10 +23,6 @@ else:  # pragma: no cover (PY3 only)
 
 
 python_token_re = re.compile(PseudoToken)
-
-SET_LOCAL = 0
-SET_GLOBAL = 1
-
 identchars = string.ascii_letters + '_'
 namechars = identchars + string.digits
 
@@ -634,12 +629,10 @@ class _LowLevelParser(SourceReader):
             elif self.matchCheetahVarStart():
                 # it has syntax that is only valid at the top level
                 self._raiseErrorAboutInvalidCheetahVarSyntaxInExpr()
+            elif self.peek() in ('{', '(', '['):
+                addBit(self.getExpression(enclosed=True))
             else:
-                token = self.getPyToken()
-                if token in ('{', '(', '['):
-                    self.rev()
-                    token = self.getExpression(enclosed=True)
-                addBit(token)
+                addBit(self.getPyToken())
 
         return ''.join(argStringBits)
 
@@ -688,12 +681,10 @@ class _LowLevelParser(SourceReader):
                 raise ParseError(self, '$ is not allowed here.')
             elif self.matchIdentifier() and not onDefVal:
                 argList.add_argument(self.getIdentifier())
+            elif onDefVal and c in ('{', '(', '['):
+                argList.add_default(self.getExpression(enclosed=True))
             elif onDefVal:
-                token = self.getPyToken()
-                if token in ('{', '(', '['):
-                    self.rev()
-                    token = self.getExpression(enclosed=True)
-                argList.add_default(token)
+                argList.add_default(self.getPyToken())
             elif c == '*' and not onDefVal:
                 varName = self.getc()
                 if self.peek() == '*':
@@ -712,17 +703,11 @@ class _LowLevelParser(SourceReader):
             enclosed=False,
             enclosures=None,  # list of tuples (char, pos), where char is ({ or [
             pyTokensToBreakAt=None,  # only works if not enclosed
-            useNameMapper=Unspecified,
     ):
         """Get a Cheetah expression that includes $CheetahVars and break at
         directive end tokens, the end of an enclosure, or at a specified
         pyToken.
         """
-
-        if useNameMapper is not Unspecified:
-            useNameMapper_orig = self.setting('useNameMapper')
-            self.setSetting('useNameMapper', useNameMapper)
-
         if enclosures is None:
             enclosures = []
 
@@ -800,15 +785,16 @@ class _LowLevelParser(SourceReader):
                 exprBits.append(token)
                 if identRE.match(token):
                     if token == 'for':
-                        expr = self.getExpression(useNameMapper=False, pyTokensToBreakAt=['in'])
+                        exprBits.append(self.getWhiteSpace())
+                        expr = self.get_python_expression(
+                            'lvalue of for must not contain a `$`',
+                            pyTokensToBreakAt=['in'],
+                        )
                         exprBits.append(expr)
                     else:
                         exprBits.append(self.getWhiteSpace())
                         if not self.atEnd() and self.peek() == '(':
                             exprBits.append(self.getCallArgString())
-
-        if useNameMapper is not Unspecified:
-            self.setSetting('useNameMapper', useNameMapper_orig)  # @@TR: see comment above
         return exprBits
 
     def getExpression(
@@ -816,15 +802,15 @@ class _LowLevelParser(SourceReader):
             enclosed=False,
             enclosures=None,  # list of tuples (char, pos), where # char is ({ or [
             pyTokensToBreakAt=None,
-            useNameMapper=Unspecified,
     ):
         """Returns the output of self.getExpressionParts() as a concatenated
         string rather than as a list.
         """
         return ''.join(self.getExpressionParts(
-            enclosed=enclosed, enclosures=enclosures,
+            enclosed=enclosed,
+            enclosures=enclosures,
             pyTokensToBreakAt=pyTokensToBreakAt,
-            useNameMapper=useNameMapper))
+        ))
 
     def get_python_expression(self, failure_msg, **kwargs):
         """Get an expression that should not contain cheetah variables.
@@ -849,7 +835,7 @@ class _LowLevelParser(SourceReader):
 
     def getPlaceholder(self, plain=False):
         startPos = self.pos()
-        lineCol = self.getRowCol(startPos)
+        lineCol = self.getRowCol()
         self.getCheetahVarStartToken()
 
         if self.peek() in '({[':
@@ -874,14 +860,14 @@ class _LowLevelParser(SourceReader):
                     assert restOfExpr[-1] == closurePairsRev[enclosureOpenChar]
                     restOfExpr = restOfExpr[:-1]
                     expr += restOfExpr
-            rawPlaceholder = self[startPos: self.pos()]
+            rawPlaceholder = self[startPos:self.pos()]
         else:
             expr = self.getExpression(enclosed=True, enclosures=enclosures)
             assert expr[-1] == closurePairsRev[enclosureOpenChar]
             expr = expr[:-1]
-            rawPlaceholder = self[startPos: self.pos()]
+            rawPlaceholder = self[startPos:self.pos()]
 
-        return (expr, rawPlaceholder, lineCol)
+        return expr, rawPlaceholder, lineCol
 
 
 class LegacyParser(_LowLevelParser):
@@ -987,14 +973,8 @@ class LegacyParser(_LowLevelParser):
         self._compiler.addComment(comm)
 
     def eatPlaceholder(self):
-        (expr, rawPlaceholder, lineCol) = self.getPlaceholder()
-
-        self._compiler.addPlaceholder(
-            expr,
-            rawPlaceholder=rawPlaceholder,
-            lineCol=lineCol,
-        )
-        return
+        expr, rawPlaceholder, lineCol = self.getPlaceholder()
+        self._compiler.addPlaceholder(expr, rawPlaceholder, lineCol)
 
     def eatPSP(self):
         self.getPSPStartToken()
@@ -1155,9 +1135,9 @@ class LegacyParser(_LowLevelParser):
         if self.matchColonForSingleLineShortFormDirective():
             self.advance()  # skip over :
             if directiveName in 'else elif except finally'.split():
-                callback(expr, dedent=False, lineCol=lineCol)
+                callback(expr, lineCol, dedent=False)
             else:
-                callback(expr, lineCol=lineCol)
+                callback(expr, lineCol)
 
             self.getWhiteSpace(maximum=1)
             self.parse(breakPoint=self.findEOL(gobble=True))
@@ -1170,7 +1150,7 @@ class LegacyParser(_LowLevelParser):
             self._eatRestOfDirectiveTag(isLineClearToStartToken, endOfFirstLinePos)
             if directiveName in self._closeableDirectives:
                 self.pushToOpenDirectivesStack(directiveName)
-            callback(expr, lineCol=lineCol)
+            callback(expr, lineCol)
 
     def eatEndDirective(self):
         isLineClearToStartToken = self.isLineClearToStartToken()
@@ -1272,7 +1252,6 @@ class LegacyParser(_LowLevelParser):
             raise ParseError(
                 self, 'Expected #def, #block or another @decorator',
             )
-        self.eatDirective()
 
     def eatDef(self):
         self._eatDefOrBlock('def')
@@ -1416,17 +1395,12 @@ class LegacyParser(_LowLevelParser):
         self._compiler.addSuper(argsList)
 
     def eatSet(self):
+        line_col = self.getRowCol()
         isLineClearToStartToken = self.isLineClearToStartToken()
         endOfFirstLine = self.findEOL()
         self.getDirectiveStartToken()
         self.advance(3)
         self.getWhiteSpace()
-        style = SET_LOCAL
-        if self.startswith('global'):
-            self.getIdentifier()
-            self.getWhiteSpace()
-            style = SET_GLOBAL
-
         lvalue = self.get_python_expression(
             'lvalue of #set cannot contain `$`',
             pyTokensToBreakAt=assignmentOps,
@@ -1437,7 +1411,7 @@ class LegacyParser(_LowLevelParser):
         self._eatRestOfDirectiveTag(isLineClearToStartToken, endOfFirstLine)
 
         expr_components = Components(lvalue, op, rvalue)
-        self._compiler.addSet(expr_components, style)
+        self._compiler.addSet(expr_components, line_col)
 
     def eatSlurp(self):
         if self.isLineClearToStartToken():
@@ -1563,7 +1537,7 @@ class LegacyParser(_LowLevelParser):
 
         if self.matchColonForSingleLineShortFormDirective():
             self.advance()  # skip over :
-            self._compiler.addIf(expr, lineCol=lineCol)
+            self._compiler.addIf(expr, lineCol)
             self.getWhiteSpace(maximum=1)
             self.parse(breakPoint=self.findEOL(gobble=True))
             self._compiler.commitStrConst()
@@ -1574,7 +1548,7 @@ class LegacyParser(_LowLevelParser):
             self.getWhiteSpace()
             self._eatRestOfDirectiveTag(isLineClearToStartToken, endOfFirstLine)
             self.pushToOpenDirectivesStack('if')
-            self._compiler.addIf(expr, lineCol=lineCol)
+            self._compiler.addIf(expr, lineCol)
 
     # end directive handlers
     def handleEndDef(self):
